/**
 * <copyright>
 *
 * Copyright (c) 2010 modelversioning.org
 * All rights reserved. This program and the accompanying materials are
 * made available under the terms of the Eclipse Public License v1.0 which
 * accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * </copyright>
 */

package org.modelversioning.operations.detection.evaluation;

import java.util.Set;

import junit.framework.Assert;
import junit.framework.TestCase;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.compare.diff.metamodel.ComparisonResourceSnapshot;
import org.eclipse.emf.compare.match.metamodel.MatchModel;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.EcoreResourceFactoryImpl;
import org.modelversioning.core.conditions.engines.UnsupportedConditionLanguage;
import org.modelversioning.core.diff.DiffSignature;
import org.modelversioning.core.diff.service.DiffService;
import org.modelversioning.core.match.MatchException;
import org.modelversioning.core.match.service.MatchService;
import org.modelversioning.ecoremutator.IModelProvider;
import org.modelversioning.ecoremutator.mutations.ModelProvider;
import org.modelversioning.operations.OperationSpecification;
import org.modelversioning.operations.detection.impl.OperationDetectionEngineImpl;
import org.modelversioning.operations.detection.operationoccurrence.OperationOccurrence;
import org.modelversioning.operations.repository.ModelOperationRepository;
import org.modelversioning.operations.repository.impl.OperationRepositoryImpl;

/**
 * @author <a href="mailto:langer@big.tuwien.ac.at">Philip Langer</a>
 * 
 */
public class ChangeSizeEvaluationRun1 extends TestCase {

	final static String opSpecPath = "models/operationSpecifications/";
	final static String evaluationRunDir = "models/change-size/";
	final static String elem1RunDir = evaluationRunDir + "1/";
	final static String elem2RunDir = evaluationRunDir + "2/";
	final static String elem3RunDir = evaluationRunDir + "3/";
	final static String elem4RunDir = evaluationRunDir + "4/";
	final static String elem5RunDir = evaluationRunDir + "5/";
	private static final int evalCount = 20;

	ResourceSet resourceSet = new ResourceSetImpl();

	final DiffService diffService = new DiffService();
	final MatchService matchService = new MatchService();

	/**
	 * The repository.
	 */
	ModelOperationRepository repository = new OperationRepositoryImpl();

	/**
	 * Operation: Ecore Inline Class
	 */
	OperationSpecification opSpecEcoreInlineClass;

	/**
	 * Operation: Ecore Pull Up Field
	 */
	OperationSpecification opSpecEcorePullUpField;

	/**
	 * Operation: Ecore Extract Class
	 */
	OperationSpecification opSpecEcoreExtractClass;

	/**
	 * Operation: Ecore Super Extract Class
	 */
	OperationSpecification opSpecEcoreExtractSuperClass;

	/**
	 * The common initial resource
	 */
	Resource initial;

	/**
	 * Evaluation resources: Change level 1
	 */

	Resource revised_1_0Ref;
	Resource revised_1_1COs;
	Resource revised_1_2COs;
	Resource revised_1_5COs;

	/**
	 * Evaluation resources: Change level 2
	 */
	Resource revised_2_0Ref;
	Resource revised_2_1COs;
	Resource revised_2_2COs;
	Resource revised_2_5COs;

	/**
	 * Evaluation resources: Change level 3
	 */
	Resource revised_3_0Ref;
	Resource revised_3_1COs;
	Resource revised_3_2COs;
	Resource revised_3_5COs;

	/**
	 * Evaluation resources: Change level 4
	 */
	Resource revised_4_0Ref;
	Resource revised_4_1COs;
	Resource revised_4_2COs;
	Resource revised_4_5COs;

	/**
	 * Evaluation resources: Change level 5
	 */
	Resource revised_5_0Ref;
	Resource revised_5_1COs;
	Resource revised_5_2COs;
	Resource revised_5_5COs;

	private OperationDetectionEngineImpl detectionEngine;

	/**
	 * {@inheritDoc}
	 * 
	 * Set up the operation repository and load sample models.
	 */
	protected void setUp() throws Exception {
		super.setUp();
		configureResourceSet();

		// load and register operation specifications
		loadOperationSpecificationEcoreInlineClass();
		loadOperationSpecificationEcorePullUpMethod();
		loadOperationSpecificationEcoreExtractClass();
		loadOperationSpecificationEcoreExtractSuperClass();
		registerToRepository(opSpecEcoreInlineClass);
		registerToRepository(opSpecEcorePullUpField);
		registerToRepository(opSpecEcoreExtractClass);
		registerToRepository(opSpecEcoreExtractSuperClass);

		// load initial model
		initial = loadResource("models/change-size/initial.ecore");
		Assert.assertTrue(initial.getContents().size() == 1);

		// Change level 1:
		revised_1_0Ref = loadResource(elem1RunDir + "revised_0.ecore");
		Assert.assertTrue(revised_1_0Ref.getContents().size() == 1);
		revised_1_1COs = loadResource(elem1RunDir + "revised_1.ecore");
		Assert.assertTrue(revised_1_1COs.getContents().size() == 1);
		revised_1_2COs = loadResource(elem1RunDir + "revised_2.ecore");
		Assert.assertTrue(revised_1_2COs.getContents().size() == 1);
		revised_1_5COs = loadResource(elem1RunDir + "revised_5.ecore");
		Assert.assertTrue(revised_1_5COs.getContents().size() == 1);

		// Change level 2:
		revised_2_0Ref = loadResource(elem2RunDir + "revised_0.ecore");
		Assert.assertTrue(revised_2_0Ref.getContents().size() == 1);
		revised_2_1COs = loadResource(elem2RunDir + "revised_1.ecore");
		Assert.assertTrue(revised_2_1COs.getContents().size() == 1);
		revised_2_2COs = loadResource(elem2RunDir + "revised_2.ecore");
		Assert.assertTrue(revised_2_2COs.getContents().size() == 1);
		revised_2_5COs = loadResource(elem2RunDir + "revised_5.ecore");
		Assert.assertTrue(revised_2_5COs.getContents().size() == 1);

		// Change level 3
		revised_3_0Ref = loadResource(elem3RunDir + "revised_0.ecore");
		Assert.assertTrue(revised_3_0Ref.getContents().size() == 1);
		revised_3_1COs = loadResource(elem3RunDir + "revised_1.ecore");
		Assert.assertTrue(revised_3_1COs.getContents().size() == 1);
		revised_3_2COs = loadResource(elem3RunDir + "revised_2.ecore");
		Assert.assertTrue(revised_3_2COs.getContents().size() == 1);
		revised_3_5COs = loadResource(elem3RunDir + "revised_5.ecore");
		Assert.assertTrue(revised_3_5COs.getContents().size() == 1);

		// Change level 4
		revised_4_0Ref = loadResource(elem4RunDir + "revised_0.ecore");
		Assert.assertTrue(revised_4_0Ref.getContents().size() == 1);
		revised_4_1COs = loadResource(elem4RunDir + "revised_1.ecore");
		Assert.assertTrue(revised_4_1COs.getContents().size() == 1);
		revised_4_2COs = loadResource(elem4RunDir + "revised_2.ecore");
		Assert.assertTrue(revised_4_2COs.getContents().size() == 1);
		revised_4_5COs = loadResource(elem4RunDir + "revised_5.ecore");
		Assert.assertTrue(revised_4_5COs.getContents().size() == 1);

		// Change level 5
		revised_5_0Ref = loadResource(elem5RunDir + "revised_0.ecore");
		Assert.assertTrue(revised_5_0Ref.getContents().size() == 1);
		revised_5_1COs = loadResource(elem5RunDir + "revised_1.ecore");
		Assert.assertTrue(revised_5_1COs.getContents().size() == 1);
		revised_5_2COs = loadResource(elem5RunDir + "revised_2.ecore");
		Assert.assertTrue(revised_5_2COs.getContents().size() == 1);
		revised_5_5COs = loadResource(elem5RunDir + "revised_5.ecore");
		Assert.assertTrue(revised_5_5COs.getContents().size() == 1);

		// initialize detection engine
		initializeDetectionEngine();
	}

	private void registerToRepository(
			OperationSpecification operationSpecification) {
		this.repository.register(operationSpecification);
	}

	private void loadOperationSpecificationEcorePullUpMethod() {
		Resource opSpecEcorePullUpFieldResource = loadResource(opSpecPath
				+ "ecore_PullUpField.operation");
		Assert
				.assertTrue(opSpecEcorePullUpFieldResource.getContents().size() > 0);
		Assert
				.assertTrue(opSpecEcorePullUpFieldResource.getContents().get(0) instanceof OperationSpecification);
		opSpecEcorePullUpField = (OperationSpecification) opSpecEcorePullUpFieldResource
				.getContents().get(0);
	}

	private void loadOperationSpecificationEcoreInlineClass() {
		Resource opSpecEcoreInlineClassResource = loadResource(opSpecPath
				+ "ecore_InlineClass.operation");
		Assert
				.assertTrue(opSpecEcoreInlineClassResource.getContents().size() > 0);
		Assert
				.assertTrue(opSpecEcoreInlineClassResource.getContents().get(0) instanceof OperationSpecification);
		opSpecEcoreInlineClass = (OperationSpecification) opSpecEcoreInlineClassResource
				.getContents().get(0);
	}

	private void loadOperationSpecificationEcoreExtractClass() {
		Resource opSpecEcoreExtractClassResource = loadResource(opSpecPath
				+ "ecore_ExtractClass.operation");
		Assert
				.assertTrue(opSpecEcoreExtractClassResource.getContents()
						.size() > 0);
		Assert
				.assertTrue(opSpecEcoreExtractClassResource.getContents()
						.get(0) instanceof OperationSpecification);
		opSpecEcoreExtractClass = (OperationSpecification) opSpecEcoreExtractClassResource
				.getContents().get(0);
	}

	private void loadOperationSpecificationEcoreExtractSuperClass() {
		Resource opSpecEcoreExtractSuperClassResource = loadResource(opSpecPath
				+ "ecore_ExtractSuperClass.operation");
		Assert.assertTrue(opSpecEcoreExtractSuperClassResource.getContents()
				.size() > 0);
		Assert.assertTrue(opSpecEcoreExtractSuperClassResource.getContents()
				.get(0) instanceof OperationSpecification);
		opSpecEcoreExtractSuperClass = (OperationSpecification) opSpecEcoreExtractSuperClassResource
				.getContents().get(0);
	}

	private void configureResourceSet() {
		// init resource set
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap()
				.put("ecore", new EcoreResourceFactoryImpl());
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap()
				.put("xmi", new EcoreResourceFactoryImpl());
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap()
				.put("operation", new EcoreResourceFactoryImpl());
	}

	private void initializeDetectionEngine() {
		// initialize detection engine
		detectionEngine = new OperationDetectionEngineImpl();
		detectionEngine.setOperationsRepository(repository);
	}

	/**
	 * Loads the resource with the specified <code>fileURI</code>.
	 * 
	 * @param fileURI
	 *            to load.
	 * @return loaded resource.
	 */
	private Resource loadResource(String fileURIString) {
		URI fileURI = URI.createURI(fileURIString, true);
		return resourceSet.getResource(fileURI, true);
	}

	public void testEvalModelSize() throws MatchException,
			UnsupportedConditionLanguage {
		System.out.println("Change size dependency evaluation");
		System.out.println("================================");
		runEval_NoRef();
		runEval_1Ref();
		runEval_2Ref();
		runEval_5Ref();
	}

	private void runEval_NoRef() throws MatchException,
			UnsupportedConditionLanguage {
		System.out.println("No refactorings");
		System.out.println("---------------");
		runNoRef_1();
		runNoRef_2();
		runNoRef_3();
		runNoRef_4();
		runNoRef_5();
	}

	private void runEval_1Ref() throws MatchException,
			UnsupportedConditionLanguage {
		System.out.println("1 refactoring");
		System.out.println("-------------");
		run1Ref_1();
		run1Ref_2();
		run1Ref_3();
		run1Ref_4();
		run1Ref_5();
	}

	private void runEval_2Ref() throws MatchException,
			UnsupportedConditionLanguage {
		System.out.println("2 refactorings");
		System.out.println("--------------");
		run2Ref_1();
		run2Ref_2();
		run2Ref_3();
		run2Ref_4();
		run2Ref_5();
	}

	private void runEval_5Ref() throws MatchException,
			UnsupportedConditionLanguage {
		System.out.println("5 refactorings");
		System.out.println("--------------");
		run5Ref_1();
		run5Ref_2();
		run5Ref_3();
		run5Ref_4();
		run5Ref_5();
	}

	private void run5Ref_5() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(this.initial,
				this.revised_5_5COs);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(5, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void run5Ref_4() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(this.initial,
				this.revised_4_5COs);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(5, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void run5Ref_3() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(this.initial,
				this.revised_3_5COs);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(5, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void run5Ref_2() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(this.initial,
				this.revised_2_5COs);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(5, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void run5Ref_1() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(this.initial,
				this.revised_1_5COs);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(5, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void run2Ref_5() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(this.initial,
				this.revised_5_2COs);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(2, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void run2Ref_4() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(this.initial,
				this.revised_4_2COs);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(2, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void run2Ref_3() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(this.initial,
				this.revised_3_2COs);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(2, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void run2Ref_2() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(this.initial,
				this.revised_2_2COs);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(2, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void run2Ref_1() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(this.initial,
				this.revised_1_2COs);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(2, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void run1Ref_5() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(this.initial,
				this.revised_5_1COs);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(1, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void run1Ref_4() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(this.initial,
				this.revised_4_1COs);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(1, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void run1Ref_3() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(this.initial,
				this.revised_3_1COs);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(1, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void run1Ref_2() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(this.initial,
				this.revised_2_1COs);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(1, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void run1Ref_1() throws MatchException,
			UnsupportedConditionLanguage {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(this.initial,
				this.revised_1_1COs);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(1, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void runNoRef_5() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(this.initial,
				this.revised_5_0Ref);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(0, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void runNoRef_4() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(this.initial,
				this.revised_4_0Ref);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(0, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void runNoRef_3() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(this.initial,
				this.revised_3_0Ref);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(0, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void runNoRef_2() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(this.initial,
				this.revised_2_0Ref);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(0, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void runNoRef_1() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(this.initial,
				this.revised_1_0Ref);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(0, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	/**
	 * Compares the specified resources.
	 * 
	 * @param resource1
	 *            origin resource to compare.
	 * @param resource2
	 *            revised resource to compare.
	 * @return comparison result.
	 * @throws MatchException
	 *             if matching fails.
	 */
	private ComparisonResourceSnapshot compare(Resource resource1,
			Resource resource2) throws MatchException {
		MatchModel matchModel = matchService.generateMatchModel(resource1,
				resource2);
		ComparisonResourceSnapshot comparisonResourceSnapshot = diffService
				.generateComparisonResourceSnapshot(matchModel);
		return comparisonResourceSnapshot;
	}
}
