/**
 * <copyright>
 *
 * Copyright (c) 2010 modelversioning.org
 * All rights reserved. This program and the accompanying materials are
 * made available under the terms of the Eclipse Public License v1.0 which
 * accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * </copyright>
 */

package org.modelversioning.operations.detection.evaluation;

import java.util.Set;

import junit.framework.Assert;
import junit.framework.TestCase;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.compare.diff.metamodel.ComparisonResourceSnapshot;
import org.eclipse.emf.compare.match.metamodel.MatchModel;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.EcoreResourceFactoryImpl;
import org.modelversioning.core.conditions.engines.UnsupportedConditionLanguage;
import org.modelversioning.core.diff.service.DiffService;
import org.modelversioning.core.match.MatchException;
import org.modelversioning.core.match.service.MatchService;
import org.modelversioning.operations.OperationSpecification;
import org.modelversioning.operations.detection.impl.OperationDetectionEngineImpl;
import org.modelversioning.operations.detection.operationoccurrence.OperationOccurrence;
import org.modelversioning.operations.repository.ModelOperationRepository;
import org.modelversioning.operations.repository.impl.OperationRepositoryImpl;

/**
 * @author <a href="mailto:langer@big.tuwien.ac.at">Philip Langer</a>
 * 
 */
public class EvaluationRun1 extends TestCase {

	final static String opSpecPath = "models/operationSpecifications/";
	final static String evaluationRunDir = "models/eval-run1/";

	ResourceSet resourceSet = new ResourceSetImpl();

	final DiffService diffService = new DiffService();
	final MatchService matchService = new MatchService();

	/**
	 * The repository.
	 */
	ModelOperationRepository repository = new OperationRepositoryImpl();

	/**
	 * Operation: Ecore Inline Class
	 */
	OperationSpecification opSpecEcoreInlineClass;

	/**
	 * Operation: Ecore Pull Up Field
	 */
	OperationSpecification opSpecEcorePullUpField;

	/**
	 * Operation: Ecore Extract Class
	 */
	OperationSpecification opSpecEcoreExtractClass;

	/**
	 * Operation: Ecore Super Extract Class
	 */
	OperationSpecification opSpecEcoreExtractSuperClass;

	/**
	 * Evaluation resources
	 */
	Resource initial;
	Resource revised;
	private OperationDetectionEngineImpl detectionEngine;

	/**
	 * {@inheritDoc}
	 * 
	 * Set up the operation repository and load sample models.
	 */
	protected void setUp() throws Exception {
		super.setUp();
		configureResourceSet();

		// load and register operation specifications
		loadOperationSpecificationEcoreInlineClass();
		loadOperationSpecificationEcorePullUpMethod();
		loadOperationSpecificationEcoreExtractClass();
		loadOperationSpecificationEcoreExtractSuperClass();
		registerToRepository(opSpecEcoreInlineClass);
		registerToRepository(opSpecEcorePullUpField);
		registerToRepository(opSpecEcoreExtractClass);
		registerToRepository(opSpecEcoreExtractSuperClass);

		// load ecore no operation sample 1
		initial = loadResource(evaluationRunDir + "initial.ecore");
		Assert.assertTrue(initial.getContents().size() == 1);
		revised = loadResource(evaluationRunDir + "revised.ecore");
		Assert.assertTrue(revised.getContents().size() == 1);
		
		// initialize detection engine
		initializeDetectionEngine();
	}

	private void registerToRepository(
			OperationSpecification operationSpecification) {
		this.repository.register(operationSpecification);
	}

	private void loadOperationSpecificationEcorePullUpMethod() {
		Resource opSpecEcorePullUpFieldResource = loadResource(opSpecPath
				+ "ecore_PullUpField.operation");
		Assert
				.assertTrue(opSpecEcorePullUpFieldResource.getContents().size() > 0);
		Assert
				.assertTrue(opSpecEcorePullUpFieldResource.getContents().get(0) instanceof OperationSpecification);
		opSpecEcorePullUpField = (OperationSpecification) opSpecEcorePullUpFieldResource
				.getContents().get(0);
	}

	private void loadOperationSpecificationEcoreInlineClass() {
		Resource opSpecEcoreInlineClassResource = loadResource(opSpecPath
				+ "ecore_InlineClass.operation");
		Assert
				.assertTrue(opSpecEcoreInlineClassResource.getContents().size() > 0);
		Assert
				.assertTrue(opSpecEcoreInlineClassResource.getContents().get(0) instanceof OperationSpecification);
		opSpecEcoreInlineClass = (OperationSpecification) opSpecEcoreInlineClassResource
				.getContents().get(0);
	}

	private void loadOperationSpecificationEcoreExtractClass() {
		Resource opSpecEcoreExtractClassResource = loadResource(opSpecPath
				+ "ecore_ExtractClass.operation");
		Assert
				.assertTrue(opSpecEcoreExtractClassResource.getContents()
						.size() > 0);
		Assert
				.assertTrue(opSpecEcoreExtractClassResource.getContents()
						.get(0) instanceof OperationSpecification);
		opSpecEcoreExtractClass = (OperationSpecification) opSpecEcoreExtractClassResource
				.getContents().get(0);
	}

	private void loadOperationSpecificationEcoreExtractSuperClass() {
		Resource opSpecEcoreExtractSuperClassResource = loadResource(opSpecPath
				+ "ecore_ExtractSuperClass.operation");
		Assert.assertTrue(opSpecEcoreExtractSuperClassResource.getContents()
				.size() > 0);
		Assert.assertTrue(opSpecEcoreExtractSuperClassResource.getContents()
				.get(0) instanceof OperationSpecification);
		opSpecEcoreExtractSuperClass = (OperationSpecification) opSpecEcoreExtractSuperClassResource
				.getContents().get(0);
	}

	private void configureResourceSet() {
		// init resource set
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap()
				.put("ecore", new EcoreResourceFactoryImpl());
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap()
				.put("xmi", new EcoreResourceFactoryImpl());
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap()
				.put("operation", new EcoreResourceFactoryImpl());
	}
	
	private void initializeDetectionEngine() {
		// initialize detection engine
		detectionEngine = new OperationDetectionEngineImpl();
		detectionEngine.setOperationsRepository(repository);
	}

	/**
	 * Loads the resource with the specified <code>fileURI</code>.
	 * 
	 * @param fileURI
	 *            to load.
	 * @return loaded resource.
	 */
	private Resource loadResource(String fileURIString) {
		URI fileURI = URI.createURI(fileURIString, true);
		return resourceSet.getResource(fileURI, true);
	}

	public void testEvalRun1() throws MatchException, UnsupportedConditionLanguage {
		for (int i = 0; i < 5; i++) {
			runEval1();
		}
	}
	
	
	public void runEval1() throws MatchException, UnsupportedConditionLanguage {
		// start detection
		System.out.println("Multiple Occurrence Sample 1");
		System.out.println("============================");

		// do the diff
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(this.initial, this.revised);
		long beforeDetection = System.currentTimeMillis();
		System.out.println("Took " + (beforeDetection - beforeCompare)
				+ " ms to diff models");

		// conduct detection
		Set<OperationOccurrence> occurrences = detectionEngine
				.findOccurrences(comparisonSnapshot);
		long afterDetection = System.currentTimeMillis();

		System.out.println("Detected Occurrences:");
		for (OperationOccurrence oc : occurrences) {
			System.out.println(oc.getAppliedOperationName() + ": "
					+ oc.getTitle());
		}

		// test occurrences
		assertEquals(5, occurrences.size());

		System.out.println("Took " + (afterDetection - beforeDetection)
				+ " ms to correctly find " + occurrences.size()
				+ " operation occurrences in sample 1");
		System.out.println();
	}

	/**
	 * Compares the specified resources.
	 * 
	 * @param resource1
	 *            origin resource to compare.
	 * @param resource2
	 *            revised resource to compare.
	 * @return comparison result.
	 * @throws MatchException
	 *             if matching fails.
	 */
	private ComparisonResourceSnapshot compare(Resource resource1,
			Resource resource2) throws MatchException {
		MatchModel matchModel = matchService.generateMatchModel(resource1,
				resource2);
		ComparisonResourceSnapshot comparisonResourceSnapshot = diffService
				.generateComparisonResourceSnapshot(matchModel);
		return comparisonResourceSnapshot;
	}

}
