/**
 * <copyright>
 *
 * Copyright (c) 2010 modelversioning.org
 * All rights reserved. This program and the accompanying materials are
 * made available under the terms of the Eclipse Public License v1.0 which
 * accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * </copyright>
 */

package org.modelversioning.operations.detection.evaluation;

import java.util.Set;

import junit.framework.Assert;
import junit.framework.TestCase;

import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.compare.diff.metamodel.ComparisonResourceSnapshot;
import org.eclipse.emf.compare.match.metamodel.MatchModel;
import org.eclipse.emf.ecore.EModelElement;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.EcoreResourceFactoryImpl;
import org.modelversioning.core.conditions.engines.UnsupportedConditionLanguage;
import org.modelversioning.core.diff.DiffSignature;
import org.modelversioning.core.diff.service.DiffService;
import org.modelversioning.core.impl.UUIDResourceFactoryImpl;
import org.modelversioning.core.match.MatchException;
import org.modelversioning.core.match.service.MatchService;
import org.modelversioning.ecoremutator.IModelProvider;
import org.modelversioning.ecoremutator.mutations.ModelProvider;
import org.modelversioning.operations.OperationSpecification;
import org.modelversioning.operations.detection.impl.OperationDetectionEngineImpl;
import org.modelversioning.operations.detection.operationoccurrence.OperationOccurrence;
import org.modelversioning.operations.repository.ModelOperationRepository;
import org.modelversioning.operations.repository.impl.OperationRepositoryImpl;

/**
 * @author <a href="mailto:langer@big.tuwien.ac.at">Philip Langer</a>
 * 
 */
public class ModelSizeEvaluationRun1 extends TestCase {

	final static String opSpecPath = "models/operationSpecifications/";
	final static String evaluationRunDir = "models/model-size/";
	final static String elem20RunDir = evaluationRunDir + "20/";
	final static String elem50RunDir = evaluationRunDir + "50/";
	final static String elem100RunDir = evaluationRunDir + "100/";
	final static String elem150RunDir = evaluationRunDir + "150/";
	final static String elem200RunDir = evaluationRunDir + "200/";
	private static final int evalCount = 20;

	ResourceSet resourceSet = new ResourceSetImpl();

	final DiffService diffService = new DiffService();
	final MatchService matchService = new MatchService();

	/**
	 * The repository.
	 */
	ModelOperationRepository repository = new OperationRepositoryImpl();

	/**
	 * Operation: Ecore Inline Class
	 */
	OperationSpecification opSpecEcoreInlineClass;

	/**
	 * Operation: Ecore Pull Up Field
	 */
	OperationSpecification opSpecEcorePullUpField;

	/**
	 * Operation: Ecore Extract Class
	 */
	OperationSpecification opSpecEcoreExtractClass;

	/**
	 * Operation: Ecore Super Extract Class
	 */
	OperationSpecification opSpecEcoreExtractSuperClass;

	/**
	 * Evaluation resources: 125 Elements
	 */
	Resource initial_20elems;
	Resource revised_20elems_0Ref;
	Resource revised_20elems_1COs;
	Resource revised_20elems_2COs;
	Resource revised_20elems_5COs;

	/**
	 * Evaluation resources: 607 Elements
	 */
	Resource initial_50elems;
	Resource revised_50elems_0Ref;
	Resource revised_50elems_1COs;
	Resource revised_50elems_2COs;
	Resource revised_50elems_5COs;

	/**
	 * Evaluation resources: XXX Elements
	 */
	Resource initial_100elems;
	Resource revised_100elems_0Ref;
	Resource revised_100elems_1COs;
	Resource revised_100elems_2COs;
	Resource revised_100elems_5COs;

	/**
	 * Evaluation resources: XXX Elements
	 */
	Resource initial_150elems;
	Resource revised_150elems_0Ref;
	Resource revised_150elems_1COs;
	Resource revised_150elems_2COs;
	Resource revised_150elems_5COs;

	/**
	 * Evaluation resources: XXX Elements
	 */
	Resource initial_200elems;
	Resource revised_200elems_0Ref;
	Resource revised_200elems_1COs;
	Resource revised_200elems_2COs;
	Resource revised_200elems_5COs;

	private OperationDetectionEngineImpl detectionEngine;

	/**
	 * {@inheritDoc}
	 * 
	 * Set up the operation repository and load sample models.
	 */
	protected void setUp() throws Exception {
		super.setUp();
		configureResourceSet();

		// load and register operation specifications
		loadOperationSpecificationEcoreInlineClass();
		loadOperationSpecificationEcorePullUpMethod();
		loadOperationSpecificationEcoreExtractClass();
		loadOperationSpecificationEcoreExtractSuperClass();
		registerToRepository(opSpecEcoreInlineClass);
		registerToRepository(opSpecEcorePullUpField);
		registerToRepository(opSpecEcoreExtractClass);
		registerToRepository(opSpecEcoreExtractSuperClass);

		// load no operation 125 elements initial model
		initial_20elems = loadResource(elem20RunDir + "initial.ecore");
		Assert.assertTrue(initial_20elems.getContents().size() == 1);
		// load no operation 125 elements revised model
		revised_20elems_0Ref = loadResource(elem20RunDir
				+ "revised_NoRefs.ecore");
		Assert.assertTrue(revised_20elems_0Ref.getContents().size() == 1);
		// load 1 operation 125 elements revised model
		revised_20elems_1COs = loadResource(elem20RunDir
				+ "revised_1Refs.ecore");
		Assert.assertTrue(revised_20elems_1COs.getContents().size() == 1);
		// load 2 operation 125 elements revised model
		revised_20elems_2COs = loadResource(elem20RunDir
				+ "revised_2Refs.ecore");
		Assert.assertTrue(revised_20elems_2COs.getContents().size() == 1);
		// load 5 operation 125 elements revised model
		revised_20elems_5COs = loadResource(elem20RunDir
				+ "revised_5Refs.ecore");
		Assert.assertTrue(revised_20elems_5COs.getContents().size() == 1);

		// load no operation 607 elements initial model
		initial_50elems = loadResource(elem50RunDir + "initial.ecore");
		Assert.assertTrue(initial_50elems.getContents().size() == 1);
		// load no operation 607 elements revised model
		revised_50elems_0Ref = loadResource(elem50RunDir
				+ "revised_NoRefs.ecore");
		Assert.assertTrue(revised_50elems_0Ref.getContents().size() == 1);
		// load 1 operation 607 elements revised model
		revised_50elems_1COs = loadResource(elem50RunDir
				+ "revised_1Refs.ecore");
		Assert.assertTrue(revised_50elems_1COs.getContents().size() == 1);
		// load 2 operation 607 elements revised model
		revised_50elems_2COs = loadResource(elem50RunDir
				+ "revised_2Refs.ecore");
		Assert.assertTrue(revised_50elems_2COs.getContents().size() == 1);
		// load 5 operation 607 elements revised model
		revised_50elems_5COs = loadResource(elem50RunDir
				+ "revised_5Refs.ecore");
		Assert.assertTrue(revised_50elems_5COs.getContents().size() == 1);

		// load no operation XXX elements initial model
		initial_100elems = loadResource(elem100RunDir + "initial.ecore");
		Assert.assertTrue(initial_100elems.getContents().size() == 1);
		// load no operation XXX elements revised model
		revised_100elems_0Ref = loadResource(elem100RunDir
				+ "revised_NoRefs.ecore");
		Assert.assertTrue(revised_100elems_0Ref.getContents().size() == 1);
		// load 1 operation XXX elements revised model
		revised_100elems_1COs = loadResource(elem100RunDir
				+ "revised_1Refs.ecore");
		Assert.assertTrue(revised_100elems_1COs.getContents().size() == 1);
		// load 2 operation XXX elements revised model
		revised_100elems_2COs = loadResource(elem100RunDir
				+ "revised_2Refs.ecore");
		Assert.assertTrue(revised_100elems_2COs.getContents().size() == 1);
		// load 5 operation XXX elements revised model
		revised_100elems_5COs = loadResource(elem100RunDir
				+ "revised_5Refs.ecore");
		Assert.assertTrue(revised_100elems_5COs.getContents().size() == 1);

		// load no operation XXX elements initial model
		initial_150elems = loadResource(elem150RunDir + "initial.ecore");
		Assert.assertTrue(initial_150elems.getContents().size() == 1);
		// load no operation XXX elements revised model
		revised_150elems_0Ref = loadResource(elem150RunDir
				+ "revised_NoRefs.ecore");
		Assert.assertTrue(revised_150elems_0Ref.getContents().size() == 1);
		// load 1 operation XXX elements revised model
		revised_150elems_1COs = loadResource(elem150RunDir
				+ "revised_1Refs.ecore");
		Assert.assertTrue(revised_150elems_1COs.getContents().size() == 1);
		// load 2 operation XXX elements revised model
		revised_150elems_2COs = loadResource(elem150RunDir
				+ "revised_2Refs.ecore");
		Assert.assertTrue(revised_150elems_2COs.getContents().size() == 1);
		// load 5 operation XXX elements revised model
		revised_150elems_5COs = loadResource(elem150RunDir
				+ "revised_5Refs.ecore");
		Assert.assertTrue(revised_150elems_5COs.getContents().size() == 1);

		// load no operation XXX elements initial model
		initial_200elems = loadResource(elem200RunDir + "initial.ecore");
		Assert.assertTrue(initial_200elems.getContents().size() == 1);
		// load no operation XXX elements revised model
		revised_200elems_0Ref = loadResource(elem200RunDir
				+ "revised_NoRefs.ecore");
		Assert.assertTrue(revised_200elems_0Ref.getContents().size() == 1);
		// load 1 operation XXX elements revised model
		revised_200elems_1COs = loadResource(elem200RunDir
				+ "revised_1Refs.ecore");
		Assert.assertTrue(revised_200elems_1COs.getContents().size() == 1);
		// load 2 operation XXX elements revised model
		revised_200elems_2COs = loadResource(elem200RunDir
				+ "revised_2Refs.ecore");
		Assert.assertTrue(revised_200elems_2COs.getContents().size() == 1);
		// load 5 operation XXX elements revised model
		revised_200elems_5COs = loadResource(elem200RunDir
				+ "revised_5Refs.ecore");
		Assert.assertTrue(revised_200elems_5COs.getContents().size() == 1);

		// initialize detection engine
		initializeDetectionEngine();
	}

	private void registerToRepository(
			OperationSpecification operationSpecification) {
		this.repository.register(operationSpecification);
	}

	private void loadOperationSpecificationEcorePullUpMethod() {
		Resource opSpecEcorePullUpFieldResource = loadResource(opSpecPath
				+ "ecore_PullUpField.operation");
		Assert.assertTrue(opSpecEcorePullUpFieldResource.getContents().size() > 0);
		Assert.assertTrue(opSpecEcorePullUpFieldResource.getContents().get(0) instanceof OperationSpecification);
		opSpecEcorePullUpField = (OperationSpecification) opSpecEcorePullUpFieldResource
				.getContents().get(0);
	}

	private void loadOperationSpecificationEcoreInlineClass() {
		Resource opSpecEcoreInlineClassResource = loadResource(opSpecPath
				+ "ecore_InlineClass.operation");
		Assert.assertTrue(opSpecEcoreInlineClassResource.getContents().size() > 0);
		Assert.assertTrue(opSpecEcoreInlineClassResource.getContents().get(0) instanceof OperationSpecification);
		opSpecEcoreInlineClass = (OperationSpecification) opSpecEcoreInlineClassResource
				.getContents().get(0);
	}

	private void loadOperationSpecificationEcoreExtractClass() {
		Resource opSpecEcoreExtractClassResource = loadResource(opSpecPath
				+ "ecore_ExtractClass.operation");
		Assert.assertTrue(opSpecEcoreExtractClassResource.getContents().size() > 0);
		Assert.assertTrue(opSpecEcoreExtractClassResource.getContents().get(0) instanceof OperationSpecification);
		opSpecEcoreExtractClass = (OperationSpecification) opSpecEcoreExtractClassResource
				.getContents().get(0);
	}

	private void loadOperationSpecificationEcoreExtractSuperClass() {
		Resource opSpecEcoreExtractSuperClassResource = loadResource(opSpecPath
				+ "ecore_ExtractSuperClass.operation");
		Assert.assertTrue(opSpecEcoreExtractSuperClassResource.getContents()
				.size() > 0);
		Assert.assertTrue(opSpecEcoreExtractSuperClassResource.getContents()
				.get(0) instanceof OperationSpecification);
		opSpecEcoreExtractSuperClass = (OperationSpecification) opSpecEcoreExtractSuperClassResource
				.getContents().get(0);
	}

	private void configureResourceSet() {
		// init resource set
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap()
				.put("ecore", new EcoreResourceFactoryImpl());
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap()
				.put("xmi", new EcoreResourceFactoryImpl());
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap()
				.put("operation", new EcoreResourceFactoryImpl());
	}

	private void initializeDetectionEngine() {
		// initialize detection engine
		detectionEngine = new OperationDetectionEngineImpl();
		detectionEngine.setOperationsRepository(repository);
	}

	/**
	 * Loads the resource with the specified <code>fileURI</code>.
	 * 
	 * @param fileURI
	 *            to load.
	 * @return loaded resource.
	 */
	private Resource loadResource(String fileURIString) {
		URI fileURI = URI.createURI(fileURIString, true);
		return resourceSet.getResource(fileURI, true);
	}

	public void testEvalModelSize() throws MatchException,
			UnsupportedConditionLanguage {
		System.out.println("Model size dependency evaluation");
		System.out.println("================================");
		runEval_NoRef();
		runEval_1Ref();
		runEval_2Ref();
		runEval_5Ref();
	}

	private void runEval_NoRef() throws MatchException,
			UnsupportedConditionLanguage {
		System.out.println("No refactorings");
		System.out.println("---------------");
		runNoRef_20();
		runNoRef_50();
		runNoRef_100();
		runNoRef_150();
		runNoRef_200();
	}

	private void runEval_1Ref() throws MatchException,
			UnsupportedConditionLanguage {
		System.out.println("1 refactoring");
		System.out.println("-------------");
		run1Ref_20();
		run1Ref_50();
		run1Ref_100();
		run1Ref_150();
		run1Ref_200();
	}

	private void runEval_2Ref() throws MatchException,
			UnsupportedConditionLanguage {
		System.out.println("2 refactorings");
		System.out.println("--------------");
		run2Ref_20();
		run2Ref_50();
		run2Ref_100();
		run2Ref_150();
		run2Ref_200();
	}

	private void runEval_5Ref() throws MatchException,
			UnsupportedConditionLanguage {
		System.out.println("5 refactorings");
		System.out.println("--------------");
		run5Ref_20();
		run5Ref_50();
		run5Ref_100();
		run5Ref_150();
		run5Ref_200();
	}

	private void run5Ref_200() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial_200elems);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(
				this.initial_200elems, this.revised_200elems_5COs);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(5, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void run5Ref_150() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial_150elems);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(
				this.initial_150elems, this.revised_150elems_5COs);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(5, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void run5Ref_100() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial_100elems);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(
				this.initial_100elems, this.revised_100elems_5COs);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(5, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void run5Ref_50() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial_50elems);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(
				this.initial_50elems, this.revised_50elems_5COs);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(5, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void run5Ref_20() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial_20elems);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(
				this.initial_20elems, this.revised_20elems_5COs);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(5, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void run2Ref_200() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial_200elems);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(
				this.initial_200elems, this.revised_200elems_2COs);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(2, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void run2Ref_150() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial_150elems);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(
				this.initial_150elems, this.revised_150elems_2COs);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(2, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void run2Ref_100() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial_100elems);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(
				this.initial_100elems, this.revised_100elems_2COs);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(2, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void run2Ref_50() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial_50elems);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(
				this.initial_50elems, this.revised_50elems_2COs);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(2, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void run2Ref_20() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial_20elems);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(
				this.initial_20elems, this.revised_20elems_2COs);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(2, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void run1Ref_200() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial_200elems);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(
				this.initial_200elems, this.revised_200elems_1COs);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(1, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void run1Ref_150() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial_150elems);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(
				this.initial_150elems, this.revised_150elems_1COs);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(1, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void run1Ref_100() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial_100elems);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(
				this.initial_100elems, this.revised_100elems_1COs);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(1, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void run1Ref_50() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial_50elems);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(
				this.initial_50elems, this.revised_50elems_1COs);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(1, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void run1Ref_20() throws MatchException,
			UnsupportedConditionLanguage {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial_20elems);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(
				this.initial_20elems, this.revised_20elems_1COs);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(1, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void runNoRef_200() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial_200elems);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(
				this.initial_200elems, this.revised_200elems_0Ref);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(0, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void runNoRef_150() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial_150elems);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(
				this.initial_150elems, this.revised_150elems_0Ref);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(0, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void runNoRef_100() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial_100elems);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(
				this.initial_100elems, this.revised_100elems_0Ref);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(0, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void runNoRef_50() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial_50elems);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(
				this.initial_50elems, this.revised_50elems_0Ref);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(0, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	private void runNoRef_20() throws UnsupportedConditionLanguage,
			MatchException {
		// output model size
		IModelProvider modelProvider = new ModelProvider();
		modelProvider.setModelResource(this.initial_20elems);
		System.out.print(modelProvider.getModelSize());
		System.out.print("\t");

		// comparison
		long beforeCompare = System.currentTimeMillis();
		ComparisonResourceSnapshot comparisonSnapshot = compare(
				this.initial_20elems, this.revised_20elems_0Ref);
		long afterCompare = System.currentTimeMillis();
		System.out.print((afterCompare - beforeCompare));
		System.out.print("\t");

		DiffSignature sig = new DiffSignature(comparisonSnapshot.getDiff());
		System.out.print(sig.getSignatureElements().size());
		System.out.print("\t");

		// conduct detection
		long evalTime = 0l;
		long conditionEvalTime = 0;
		long preselectionTime = 0;
		for (int i = 0; i < evalCount; i++) {
			initializeDetectionEngine();
			long beforeDetection = System.currentTimeMillis();
			Set<OperationOccurrence> occurrences = detectionEngine
					.findOccurrences(comparisonSnapshot);
			long afterDetection = System.currentTimeMillis();
			evalTime += (afterDetection - beforeDetection);
			conditionEvalTime += detectionEngine.getEvaluationTime();
			preselectionTime += detectionEngine.getPreselectionTime();
			// test occurrences
			assertEquals(0, occurrences.size());
		}
		evalTime = evalTime / evalCount;

		// output spent time
		System.out.print(evalTime + "\t" + (preselectionTime / evalCount)
				+ "\t" + (conditionEvalTime / evalCount));
		System.out.println();
	}

	/**
	 * Compares the specified resources.
	 * 
	 * @param resource1
	 *            origin resource to compare.
	 * @param resource2
	 *            revised resource to compare.
	 * @return comparison result.
	 * @throws MatchException
	 *             if matching fails.
	 */
	private ComparisonResourceSnapshot compare(Resource resource1,
			Resource resource2) throws MatchException {
		MatchModel matchModel = matchService.generateMatchModel(resource1,
				resource2);
		ComparisonResourceSnapshot comparisonResourceSnapshot = diffService
				.generateComparisonResourceSnapshot(matchModel);
		return comparisonResourceSnapshot;
	}

	/**
	 * Counts the contents of the specified <code>iterator</code>.
	 * 
	 * @param iterator
	 *            to count objects of.
	 * @return the size of all objects held by the iterator.
	 */
	private int count(TreeIterator<Object> iterator) {
		int size = 0;
		while (iterator.hasNext()) {
			if (iterator.next() instanceof EModelElement) {
				size++;
			}
		}
		return size;
	}

}
